@@ -1,23 +1,37 @@
+"""
+Stream V1
+The new Alpaca data stream is under stream.py
+"""
 import asyncio
+import deprecation
 import json
 import os
 import re
 import traceback
 from asyncio import CancelledError
+import queue
 
 import websockets
 from .common import get_base_url, get_data_url, get_credentials, URL
 from .entity import Account, Entity, trade_mapping, agg_mapping, quote_mapping
-from . import polygon
 from .entity import Trade, Quote, Agg
 import logging
 from typing import List, Callable
 
 
 class _StreamConn(object):
-    def __init__(self, key_id: str, secret_key: str, base_url: URL):
+    def __init__(self, key_id: str,
+                 secret_key: str,
+                 base_url: URL,
+                 oauth: str = None,
+                 raw_data: bool = False):
+        """
+        :param raw_data: should we return stream data raw or wrap it with
+                         Entity objects.
+        """
         self._key_id = key_id
         self._secret_key = secret_key
+        self._oauth = oauth
         self._base_url = re.sub(r'^http', 'ws', base_url)
         self._endpoint = self._base_url + '/stream'
         self._handlers = {}
@@ -28,16 +42,21 @@ class _StreamConn(object):
         self._retry_wait = int(os.environ.get('APCA_RETRY_WAIT', 3))
         self._retries = 0
         self._consume_task = None
+        self._raw_data = raw_data
 
     async def _connect(self):
-        ws = await websockets.connect(self._endpoint)
-        await ws.send(json.dumps({
+        message = {
             'action': 'authenticate',
-            'data': {
-                'key_id': self._key_id,
+            'data':   {
+                'oauth_token': self._oauth
+            } if self._oauth else {
+                'key_id':     self._key_id,
                 'secret_key': self._secret_key,
             }
-        }))
+        }
+
+        ws = await websockets.connect(self._endpoint)
+        await ws.send(json.dumps(message))
         r = await ws.recv()
         if isinstance(r, bytes):
             r = r.decode('utf-8')
@@ -109,7 +128,7 @@ class _StreamConn(object):
             self._streams |= set(channels)
             await self._ws.send(json.dumps({
                 'action': 'listen',
-                'data': {
+                'data':   {
                     'streams': channels,
                 }
             }))
@@ -120,18 +139,21 @@ class _StreamConn(object):
         if len(channels) > 0:
             await self._ws.send(json.dumps({
                 'action': 'unlisten',
-                'data': {
+                'data':   {
                     'streams': channels,
                 }
             }))
 
     async def close(self):
-        if self._consume_task:
-            self._consume_task.cancel()
+        await self.cancel_task()
         if self._ws:
             await self._ws.close()
             self._ws = None
 
+    async def cancel_task(self):
+        if self._consume_task:
+            self._consume_task.cancel()
+
     def _cast(self, channel, msg):
         if channel == 'account_updates':
             return Account(msg)
@@ -151,8 +173,11 @@ class _StreamConn(object):
     async def _dispatch(self, channel, msg):
         for pat, handler in self._handlers.items():
             if pat.match(channel):
-                ent = self._cast(channel, msg['data'])
-                await handler(self, channel, ent)
+                if self._raw_data:
+                    await handler(self, channel, msg)
+                else:
+                    ent = self._cast(channel, msg['data'])
+                    await handler(self, channel, ent)
 
     def on(self, channel_pat, symbols=None):
         def decorator(func):
@@ -176,8 +201,8 @@ class _StreamConn(object):
         del self._handlers[channel_pat]
 
 
+@deprecation.deprecated(deprecated_in="v1.0.0", details="Use Stream instead")
 class StreamConn(object):
-
     def __init__(
             self,
             key_id: str = None,
@@ -185,13 +210,23 @@ class StreamConn(object):
             base_url: URL = None,
             data_url: URL = None,
             data_stream: str = None,
-            debug: bool = False
-    ):
-        self._key_id, self._secret_key, _ = get_credentials(key_id, secret_key)
+            debug: bool = False,
+            oauth: str = None,
+            raw_data: bool = False):
+        """
+        :param base_url: api.alpaca.markets
+        :param data_url: data.alpaca.markets
+        :param data_stream: alpacadatav1
+        :param debug: should print exceptions?
+        :param raw_data: should we return stream data raw or wrap it with
+                         Entity objects.
+        """
+        self._key_id, self._secret_key, self._oauth = \
+            get_credentials(key_id, secret_key, oauth)
         self._base_url = base_url or get_base_url()
         self._data_url = data_url or get_data_url()
         if data_stream is not None:
-            if data_stream in ('alpacadatav1', 'polygon'):
+            if data_stream in ('alpacadatav1'):
                 _data_stream = data_stream
             else:
                 raise ValueError('invalid data_stream name {}'.format(
@@ -200,22 +235,22 @@ class StreamConn(object):
             _data_stream = 'alpacadatav1'
         self._data_stream = _data_stream
         self._debug = debug
+        self._raw_data = raw_data
+        self._stop_stream_queue = queue.Queue()
 
         self.trading_ws = _StreamConn(self._key_id,
                                       self._secret_key,
-                                      self._base_url)
+                                      self._base_url,
+                                      self._oauth,
+                                      raw_data=self._raw_data)
 
-        if self._data_stream == 'polygon':
-            self.data_ws = polygon.StreamConn(
-                self._key_id + '-staging' if 'staging' in self._base_url else
-                self._key_id)
-            self._data_prefixes = (('Q.', 'T.', 'A.', 'AM.'))
-        else:
-            self.data_ws = _StreamConn(self._key_id,
-                                       self._secret_key,
-                                       self._data_url)
-            self._data_prefixes = (
-                ('Q.', 'T.', 'AM.', 'alpacadatav1/'))
+        self.data_ws = _StreamConn(self._key_id,
+                                   self._secret_key,
+                                   self._data_url,
+                                   self._oauth,
+                                   raw_data=self._raw_data)
+        self._data_prefixes = (
+            ('Q.', 'T.', 'AM.', 'alpacadatav1/'))
 
         self._handlers = {}
         self._handler_symbols = {}
@@ -299,6 +334,9 @@ class StreamConn(object):
                 logging.error(f"error while consuming ws messages: {m}")
                 if self._debug:
                     traceback.print_exc()
+                if not self._stop_stream_queue.empty():
+                    self._stop_stream_queue.get()
+                    should_renew = False
                 loop.run_until_complete(self.close(should_renew))
                 if loop.is_running():
                     loop.close()
@@ -317,15 +355,26 @@ class StreamConn(object):
         if renew:
             self.trading_ws = _StreamConn(self._key_id,
                                           self._secret_key,
-                                          self._base_url)
-            if self._data_stream == 'polygon':
-                self.data_ws = polygon.StreamConn(
-                    self._key_id + '-staging' if 'staging' in
-                    self._base_url else self._key_id)
-            else:
-                self.data_ws = _StreamConn(self._key_id,
-                                           self._secret_key,
-                                           self._data_url)
+                                          self._base_url,
+                                          self._oauth,
+                                          raw_data=self._raw_data)
+            self.data_ws = _StreamConn(self._key_id,
+                                       self._secret_key,
+                                       self._data_url,
+                                       self._oauth,
+                                       raw_data=self._raw_data)
+
+    async def stop_ws(self):
+        """
+        Signal the ws connections to stop listenning to api stream.
+        """
+        self._stop_stream_queue.put_nowait({"should_stop": True})
+        if self.trading_ws is not None:
+            logging.info("Stopping the trading websocket connection")
+            await self.trading_ws.cancel_task()
+        if self.data_ws is not None:
+            logging.info("Stopping the data websocket connection")
+            await self.data_ws.cancel_task()
 
     def on(self, channel_pat, symbols=None):
         def decorator(func):
